@@ -1,10 +1,14 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 from itertools import chain
+
+from sql import Literal, For
+
 from trytond.model import Workflow, ModelView, ModelSQL, fields
-from trytond.pyson import Equal, Eval, If, In, Get
+from trytond.pyson import Eval, If
 from trytond.transaction import Transaction
 from trytond.pool import Pool
+from trytond.tools import grouped_slice
 
 __all__ = ['Period', 'Cache']
 
@@ -12,13 +16,12 @@ __all__ = ['Period', 'Cache']
 class Period(Workflow, ModelSQL, ModelView):
     'Stock Period'
     __name__ = 'stock.period'
-    _rec_name = 'date'
     date = fields.Date('Date', required=True, states={
-        'readonly': Equal(Eval('state'), 'closed'),
-        }, depends=['state'])
+            'readonly': Eval('state') == 'closed',
+            }, depends=['state'])
     company = fields.Many2One('company.company', 'Company', required=True,
         domain=[
-            ('id', If(In('company', Eval('context', {})), '=', '!='),
+            ('id', If(Eval('context', {}).contains('company'), '=', '!='),
                 Eval('context', {}).get('company', -1)),
             ])
     caches = fields.One2Many('stock.period.cache', 'period', 'Caches',
@@ -45,12 +48,18 @@ class Period(Workflow, ModelSQL, ModelView):
         cls._buttons.update({
                 'draft': {
                     'invisible': Eval('state') == 'draft',
+                    'depends': ['state'],
                     },
                 'close': {
                     'invisible': Eval('state') == 'closed',
+                    'depends': ['state'],
                     },
                 })
 
+    @staticmethod
+    def default_company():
+        return Transaction().context.get('company')
+
     @staticmethod
     def default_state():
         return 'draft'
@@ -62,21 +71,23 @@ class Period(Workflow, ModelSQL, ModelView):
     @staticmethod
     def get_cache(grouping):
         pool = Pool()
-        if grouping == ('product',):
+        if all(g == 'product' or g.startswith('product.') for g in grouping):
             return pool.get('stock.period.cache')
 
+    def get_rec_name(self, name):
+        return str(self.date)
+
     @classmethod
     @ModelView.button
     @Workflow.transition('draft')
     def draft(cls, periods):
-        in_max = Transaction().cursor.IN_MAX
         for grouping in cls.groupings():
             Cache = cls.get_cache(grouping)
             caches = []
-            for i in xrange(0, len(periods), in_max):
+            for sub_periods in grouped_slice(periods):
                 caches.append(Cache.search([
                             ('period', 'in',
-                                [p.id for p in periods[i:i + in_max]]),
+                                [p.id for p in sub_periods]),
                             ], order=[]))
             Cache.delete(list(chain(*caches)))
 
@@ -89,6 +100,20 @@ class Period(Workflow, ModelSQL, ModelView):
         Location = pool.get('stock.location')
         Move = pool.get('stock.move')
         Date = pool.get('ir.date')
+        transaction = Transaction()
+        connection = transaction.connection
+        database = transaction.database
+
+        # XXX: A move in the period could be inserted before the lock
+        # from a different transaction. It will not be taken in the pbl
+        # computation but it is quite rare because only past periods are
+        # closed.
+        database.lock(connection, Move._table)
+        if database.has_select_for():
+            move = Move.__table__()
+            query = move.select(Literal(1), for_=For('UPDATE', nowait=True))
+            with connection.cursor() as cursor:
+                cursor.execute(*query)
 
         locations = Location.search([
                 ('type', 'not in', ['warehouse', 'view']),
@@ -121,7 +146,7 @@ class Period(Workflow, ModelSQL, ModelView):
                         ):
                     pbl = Product.products_by_location(
                         [l.id for l in locations], grouping=grouping)
-                for key, quantity in pbl.iteritems():
+                for key, quantity in pbl.items():
                     values = {
                         'location': key[0],
                         'period': period.id,
